PROJECT_NAME = 'quintagroup.seoptimizer'
SKINS_DIR = 'skins'

GLOBALS = globals()

PROPERTY_SHEET = 'seo_properties'
SHEET_TITLE = 'SEO Properties'

STOP_WORDS = ['a', 'an', 'amp', 'and', 'are', 'arial', 'as', 'at', 'be', 'but', 'by', 'can', 'com', 'do', 'font',
              'for', 'from', 'gif', 'had', 'has', 'have', 'he', 'helvetica', 'her', 'his', 'how', 'href', 'i',
              'if', 'in', 'is', 'it', 'javascript', 'jpg', 'made', 'net', 'of', 'on', 'or', 'org', 'our', 'sans',
              'see', 'serif', 'she', 'that', 'the', 'this', 'to', 'us', 'we', 'with', 'you', 'your']

FIELDS = ['seo_title', 'seo_description', 'seo_keywords']

DEFAULT_CUSTOM_METATAGS = []



import re
from base import getToolByName, FunctionalTestCase, newSecurityManager
from config import *

class TestBaseURL(FunctionalTestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct(PROJECT_NAME)
        #self.portal.changeSkin('Plone Default')

        self.basic_auth = 'portal_manager:secret'
        uf = self.app.acl_users
        uf.userFolderAddUser('portal_manager', 'secret', ['Manager'], [])
        user = uf.getUserById('portal_manager')
        if not hasattr(user, 'aq_base'):
            user = user.__of__(uf)
        newSecurityManager(None, user)

    def test_notFolderBaseURL(self):
        my_doc = self.portal.invokeFactory('Document', id='my_doc')
        my_doc = self.portal['my_doc']
        regen = re.compile('<base\s+[^>]*href=\"([^\"]*)\"[^>]*>', re.S|re.M)

        path = "/%s" % my_doc.absolute_url(1)
        html = self.publish(path, self.basic_auth).getBody()
        burls = regen.findall(html)

        mydocurl = my_doc.absolute_url()
        self.assert_(not [1 for burl in burls if not burl==mydocurl],
           "Wrong BASE URL for document: %s, all must be: %s" % (burls, mydocurl))

    def test_folderBaseURL(self):
        my_fldr = self.portal.invokeFactory('Folder', id='my_fldr')
        my_fldr = self.portal['my_fldr']
        regen = re.compile('<base\s+[^>]*href=\"([^\"]*)\"[^>]*>', re.S|re.M)
        
        path = "/%s" % my_fldr.absolute_url(1)
        html = self.publish(path, self.basic_auth).getBody()
        burls = regen.findall(html)

        myfldrurl = my_fldr.absolute_url() + '/'
        self.assert_(not [1 for burl in burls if not burl==myfldrurl],
           "Wrong BASE URL for folder: %s , all must be : %s" % (burls, myfldrurl))

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBaseURL))
    return suite



from AccessControl import allow_module
from zope.component import queryMultiAdapter
from zope.i18nmessageid import MessageFactory

from Acquisition import aq_inner
from DateTime import DateTime

from Products.CMFCore.utils import getToolByName

from quintagroup.seoptimizer.interfaces import IMetaKeywords, IMappingMetaTags
from quintagroup.seoptimizer.util import SortedDict

SeoptimizerMessageFactory = MessageFactory('quintagroup.seoptimizer')

allow_module('quintagroup.seoptimizer.util')
qSEO_globals = globals()


try:
    from Products.CMFPlone.PloneTool import PloneTool, METADATA_DCNAME, \
        FLOOR_DATE, CEILING_DATE
    _present = hasattr(PloneTool, "listMetaTags")
except ImportError:
    _present = False


if _present:
    old_lmt = PloneTool.listMetaTags

    def listMetaTags(self, context):
        """Lists meta tags helper.

        Creates a mapping of meta tags.
        """

        from quintagroup.seoptimizer.browser.interfaces import IPloneSEOLayer
        if not IPloneSEOLayer.providedBy(self.REQUEST):
            return old_lmt(getToolByName(self, 'plone_utils'), context)

        result = SortedDict()
        site_props = getToolByName(self, 'portal_properties').site_properties
        use_all = site_props.getProperty('exposeDCMetaTags', None)

        seo_context = queryMultiAdapter((context, self.REQUEST), name='seo_context')
        adapter = IMappingMetaTags(context, None)
        mapping_metadata = adapter and adapter.getMappingMetaTags() or SortedDict()

        if not use_all:
            metadata_names = mapping_metadata.has_key('DC.description') and {'DC.description': mapping_metadata['DC.description']} or SortedDict()
            if mapping_metadata.has_key('description'):
                metadata_names['description'] = mapping_metadata['description']
        else:
            metadata_names = mapping_metadata

        for key, accessor in metadata_names.items():
            if accessor == 'meta_keywords':
                # Render all the existing keywords for the current content type
                adapter = IMetaKeywords(context, None)
                if adapter is not None:
                    keywords = adapter.getMetaKeywords()
                    if keywords:
                        result['keywords'] = keywords
                continue

            method = getattr(seo_context, accessor, None)
            if method is None:
                method = getattr(aq_inner(context).aq_explicit, accessor, None)

            if not callable(method):
                continue

            # Catch AttributeErrors raised by some AT applications
            try:
                value = method()
            except AttributeError:
                value = None

            if not value:
                # No data
                continue
            if accessor == 'Publisher' and value == 'No publisher':
                # No publisher is hardcoded (TODO: still?)
                continue
            if isinstance(value, (list, tuple)):
                # convert a list to a string
                value = ', '.join(value)

            # Special cases
            if accessor == 'Description' and not metadata_names.has_key('description'):
                result['description'] = value
            elif accessor == 'Subject' and not metadata_names.has_key('keywords'):
                result['keywords'] = value

            if accessor not in ('Description', 'Subject'):
                result[key] = value

        if use_all:
            created = context.CreationDate()

            try:
                effective = context.EffectiveDate()
                if effective == 'None':
                    effective = None
                if effective:
                    effective = DateTime(effective)
            except AttributeError:
                effective = None

            try:
                expires = context.ExpirationDate()
                if expires == 'None':
                    expires = None
                if expires:
                    expires = DateTime(expires)
            except AttributeError:
                expires = None

            # Filter out DWIMish artifacts on effective / expiration dates
            if effective is not None and \
               effective > FLOOR_DATE and \
               effective != created:
                eff_str = effective.Date()
            else:
                eff_str = ''

            if expires is not None and expires < CEILING_DATE:
                exp_str = expires.Date()
            else:
                exp_str = ''

            if exp_str or exp_str:
                result['DC.date.valid_range'] = '%s - %s' % (eff_str, exp_str)

        # add custom meta tags (added from qseo tab by user) for given context and default from configlet
        custom_meta_tags = seo_context and seo_context.seo_customMetaTags() or []
        for tag in custom_meta_tags:
            if tag['meta_content']:
                result[tag['meta_name']] = tag['meta_content']

        return result

    PloneTool.listMetaTags = listMetaTags



import re, commands
from zope.interface import implements
from zope.component import queryMultiAdapter
from Products.CMFCore.utils import getToolByName

from quintagroup.seoptimizer.interfaces import IMetaKeywords, IMappingMetaTags
from quintagroup.seoptimizer.util import SortedDict


class MetaKeywordsAdapter(object):
    implements(IMetaKeywords)

    def __init__(self, context):
        self.context = context

    def getMetaKeywords(self):
        """ See interface.
        """
        request = self.context.REQUEST
        meta_keywords = ''
        seo_context = queryMultiAdapter((self.context, request), name='seo_context')
        if seo_context:
            meta_keywords = list(seo_context.meta_keywords())
        return ', '.join(meta_keywords)


class MappingMetaTags(object):
    implements(IMappingMetaTags)

    def __init__(self, context):
        self.context = context
        self.portal_props = getToolByName(self.context, 'portal_properties')
        self.seo_props = getToolByName(self.portal_props, 'seo_properties', None)

    def getMappingMetaTags(self):
        """ See interface.
        """
        metadata_name = SortedDict()
        if self.seo_props:
            pmn = self.seo_props.getProperty('metatags_order')
            pmn = pmn and pmn or ''
            for mt in [mt.split(' ') for mt in pmn if len(mt.split(' '))==2]:
                metadata_name[mt[0]] = mt[1]
        return metadata_name



#
# Test product's installation
#
import string
from base import getToolByName, FunctionalTestCase, TestCase, newSecurityManager
from config import *


class TestBeforeInstall(FunctionalTestCase):

    def afterSetUp(self):
        self.basic_auth = 'mgr:mgrpw'
        self.portal_path = '/%s' % self.portal.absolute_url(1)

    def testAccessPortalRootAnonymous(self):
        response = self.publish(self.portal_path)
        self.assertEqual(response.getStatus(), 200)

    def testAccessPortalRootAuthenticated(self):
        response = self.publish(self.portal_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)


class TestInstallation(TestCase):

    def afterSetUp(self):
        self.properties = getToolByName(self.portal, 'portal_properties')
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct(PROJECT_NAME)

    def testAddingPropertySheet(self):
        """ Test adding property sheet to portal_properties tool """
        self.failUnless(hasattr(self.properties.aq_base, PROPERTY_SHEET))

    def testAddingPropertyFields(self):
        """ Test adding property field to portal_properties.maps_properties sheet """
        map_sheet = self.properties[PROPERTY_SHEET]
        for key, value in PROPS.items():
            self.failUnless(map_sheet.hasProperty(key) and list(map_sheet.getProperty(key)) == value)

    def test_configlet_install(self):
        configTool = getToolByName(self.portal, 'portal_controlpanel', None)
        self.assert_(PROJECT_NAME in [a.getId() for a in configTool.listActions()], 'Configlet not found')

    def test_actions_install(self):
        portal_types = getToolByName(self.portal, 'portal_types')
        for ptype in portal_types.objectValues():
            try:
                #for Plone-2.5 and higher
                acts = filter(lambda x: x.id == 'seo_properties', ptype.listActions())
                action = acts and acts[0] or None
            except AttributeError:
                action = ptype.getActionById('seo_properties', default=None )

            if ptype.getId() in qSEO_TYPES:
                self.assert_(action, 'Action for %s not found' % ptype.getId())
            else:
                self.assert_(not action, 'Action found for %s' % ptype.getId())

    def test_skins_install(self):
        skinstool=getToolByName(self.portal, 'portal_skins')

        for skin in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skin)
            path = map( string.strip, string.split( path,',' ) )
            self.assert_(PROJECT_NAME in path, 'qSEOptimizer layer not found in %s' %skin)

    def test_versionedskin_install(self):
        skinstool=getToolByName(self.portal, 'portal_skins')
        mtool = getToolByName(self.portal, 'portal_migration')
        plone_version = mtool.getFileSystemVersion()
        if plone_version < "3":
            for skin in skinstool.getSkinSelections():
                path = skinstool.getSkinPath(skin)
                path = map( string.strip, string.split( path,',' ) )
                self.assert_(PROJECT_NAME+'/%s' % plone_version in path, 'qSEOptimizer versioned layer not found in %s' %skin)

    def test_actions_uninstall(self):
        self.qi.uninstallProducts([PROJECT_NAME])
        self.assertNotEqual(self.qi.isProductInstalled(PROJECT_NAME), True,'qSEOptimizer is already installed')
        portal_types = getToolByName(self.portal, 'portal_types')
        for ptype in portal_types.objectValues():
            try:
                #for Plone-2.5 and higher
                acts = filter(lambda x: x.id == 'seo_properties', ptype.listActions())
                action = acts and acts[0] or None
            except AttributeError:
                action = ptype.getActionById('seo_properties', default=None )

            self.assert_(not action, 'Action for %s found after uninstallation' % ptype.getId())

    def test_skins_uninstall(self):
        self.qi.uninstallProducts([PROJECT_NAME])
        self.assertNotEqual(self.qi.isProductInstalled(PROJECT_NAME), True,'qSEOptimizer is already installed')
        skinstool=getToolByName(self.portal, 'portal_skins')

        for skin in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skin)
            path = map( string.strip, string.split( path,',' ) )
            self.assert_(not PROJECT_NAME in path, 'qSEOptimizer layer found in %s after uninstallation' %skin)

    def test_versionedskin_uninstall(self):
        self.qi.uninstallProducts([PROJECT_NAME])
        self.assertNotEqual(self.qi.isProductInstalled(PROJECT_NAME), True,'qSEOptimizer is already installed')
        skinstool=getToolByName(self.portal, 'portal_skins')
        mtool = getToolByName(self.portal, 'portal_migration')
        plone_version = mtool.getFileSystemVersion()

        for skin in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skin)
            path = map( string.strip, string.split( path,',' ) )
            self.assert_(not PROJECT_NAME+'/%s' % plone_version in path, 'qSEOptimizer versioned layer found in %s after uninstallation' %skin)

    def test_configlet_uninstall(self):
        self.qi.uninstallProducts([PROJECT_NAME])
        self.assertNotEqual(self.qi.isProductInstalled(PROJECT_NAME), True,'qSEOptimizer is already installed')

        configTool = getToolByName(self.portal, 'portal_controlpanel', None)
        self.assert_(not PROJECT_NAME in [a.getId() for a in configTool.listActions()], 'Configlet found after uninstallation')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBeforeInstall))
    suite.addTest(makeSuite(TestInstallation))
    return suite



from Products.CMFCore.utils import getToolByName
from quintagroup.seoptimizer.config import PROJECT_NAME

def install(portal, reinstall=False):
    setup_tool = getToolByName(portal, 'portal_setup')
    setup_tool.setBaselineContext('profile-%s:uninstall'%PROJECT_NAME)
    if reinstall:
        setup_tool.setBaselineContext('profile-%s:reinstall'%PROJECT_NAME)
        setup_tool.runAllImportStepsFromProfile('profile-%s:reinstall'%PROJECT_NAME)
        return "Ran reinstall steps."
    else:
        setup_tool.setBaselineContext('profile-%s:uninstall'%PROJECT_NAME)
        setup_tool.runAllImportStepsFromProfile('profile-%s:default'%PROJECT_NAME)
        return "Ran all install steps."

def uninstall(portal, reinstall=False):
    setup_tool = getToolByName(portal, 'portal_setup')
    setup_tool.setBaselineContext('profile-%s:uninstall'%PROJECT_NAME)
    if reinstall:
        return "Ran reinstall steps."
    else:
        setup_tool.runAllImportStepsFromProfile('profile-%s:uninstall'%PROJECT_NAME)
        return "Ran all uninstall steps."



import logging
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import Expression

logger = logging.getLogger('quintagroup.seoptimizer')

def migrationActions(site):
    old = 'qseo_properties_edit_form'
    new = '@@seo-context-properties'
    condition = "python:exists('portal/@@seo-context-properties')"
    for ptype in site.portal_types.objectValues():
        acts = filter(lambda x: x.id == 'seo_properties' , ptype.listActions())
        for act in acts:
            log = 0
            if not act.condition:
                act.condition = Expression(condition)
                log = 1
            ac_exp = act.getActionExpression()
            if old in ac_exp:
                act.setActionExpression(ac_exp.replace(old, new))
                log = 1
            if log:
                logger.log(logging.INFO, "Updated \"SEO Properties\" action in %s type." % ptype.id)

def removeSkin(self, layer):
    """ Remove layers.
    """
    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        original_path = skinstool.getSkinPath(skinName)
        original_path = [l.strip() for l in original_path.split(',')]
        new_path= []
        for l in original_path:
            if (l == layer) or (l.startswith(layer+'/')):
                logger.log(logging.INFO, "Removed %s layer from %s skin." % (l, skinName))
                continue
            new_path.append(l)
        skinstool.addSkinSelection(skinName, ','.join(new_path))

def removeActions(self):
    """ Remove actions.
    """
    tool = getToolByName(self, 'portal_types')
    for ptype in tool.objectValues():
        if ptype.getId() in ['File','Document','News Item']:
            acts = filter(lambda x: x.id == 'seo_properties', ptype.listActions())
            action = acts and acts[0] or None
            if action != None:
                acts = list(ptype.listActions())
                ptype.deleteActions([acts.index(a) for a in acts if a.getId()=='seo_properties'])
                logger.log(logging.INFO, "Deleted \"SEO Properties\" action for %s type." % ptype.id)

def remove_configlets(context, conf_ids):
    """ Remove configlets.
    """
    configTool = getToolByName(context, 'portal_controlpanel', None)
    if configTool:
        for id in conf_ids:
            configTool.unregisterConfiglet(id)
            logger.log(logging.INFO, "Unregistered \"%s\" configlet." % id)

def importVarious(context):
    """ Do customized installation.
    """
    if context.readDataFile('seo_install.txt') is None:
        return

def reinstall(context):
    """ Do customized reinstallation.
    """
    if context.readDataFile('seo_reinstall.txt') is None:
        return
    site = context.getSite()
    migrationActions(site)

def uninstall(context):
    """ Do customized uninstallation.
    """
    if context.readDataFile('seo_uninstall.txt') is None:
        return
    site = context.getSite()
    removeSkin(site, 'quintagroup.seoptimizer' )
    removeActions(site)
    remove_configlets(site, ('quintagroup.seoptimizer',))



import urllib, re
from Acquisition import aq_inner
from base import getToolByName, FunctionalTestCase, newSecurityManager
from config import *

class TestUsageKeywords(FunctionalTestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct(PROJECT_NAME)
        self.sp = self.portal.portal_properties.seo_properties
        self.pu = self.portal.plone_utils

        self.basic_auth = 'portal_manager:secret'
        uf = self.app.acl_users
        uf.userFolderAddUser('portal_manager', 'secret', ['Manager'], [])
        user = uf.getUserById('portal_manager')
        if not hasattr(user, 'aq_base'):
            user = user.__of__(uf)
        newSecurityManager(None, user)

        '''Preparation for functional testing'''
        self.my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']

    def test_default_values_radiobutton_in_configlet(self):
        self.assertEqual(self.sp.getProperty('settings_use_keywords_sg', 0), 3)
        self.assertEqual(self.sp.getProperty('settings_use_keywords_lg', 0), 2)

    def test_changes_using_keywords_in_configlet(self):
        for sg, lg in ((1, 1), (1, 2), (2, 1), (2, 2), (3, 1), (3, 2)):
            path = self.portal.id+'/@@seo-controlpanel?settingsUseKeywordsSG=%s'\
                                '&settingsUseKeywordsLG=%s&form.submitted=1' % (sg, lg)
            self.publish(path, self.basic_auth)
            self.assertEqual(self.sp.getProperty('settings_use_keywords_sg', 0), sg)
            self.assertEqual(self.sp.getProperty('settings_use_keywords_lg', 0), lg)

    def test_additional_keywords_in_configlet(self):
        quoted_keywords = urllib.quote('foo\nbar')
        path = self.portal.id+'/@@seo-controlpanel?additionalKeywords:lines=%s&form.submitted=1'%quoted_keywords
        self.publish(path, self.basic_auth)
        self.assertEqual(self.sp.additional_keywords, ('foo', 'bar'))
        self.publish(self.portal.id+'/@@seo-controlpanel?form.submitted=1', self.basic_auth)
        self.assertEqual(self.sp.additional_keywords, ())

    def test_listMetaTags_empty(self):
        metatags = self.pu.listMetaTags(self.my_doc)
        self.assert_('keywords' not in metatags)

    def test_listMetaTags_one(self):
        self.my_doc.setText('<p>foo</p>')
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        self.my_doc.manage_addProperty('qSEO_keywords', ('foo',), 'lines')
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="foo"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, "No 'foo' keyword find")

    def test_listMetaTags_two(self):
        self.my_doc.setText('<p>foo bar</p>')
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        self.my_doc.manage_addProperty('qSEO_keywords', ('foo', 'bar'), 'lines')
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:foo|bar),\s*(?:foo|bar)"\s*)){2}/>)',
                     self.html, re.S|re.M)
        self.assert_(m, "No 'foo, bar' keyword find")

    def test_additional_keywords_in_listMetaTags_empty(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        self.sp.additional_keywords = ('foo',)
        metatags = self.pu.listMetaTags(self.my_doc)
        self.assert_('keywords' not in metatags)

    def test_additional_keywords_in_listMetaTags_one(self):
        self.my_doc.setText('<p>foo</p>')
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        self.sp.additional_keywords = ('foo',)
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="foo"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, "No 'foo' keyword find")

    def test_additional_keywords_in_listMetaTags_two(self):
        self.my_doc.setText('<p>foo bar</p>')
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        self.sp.additional_keywords = ('foo', 'bar')
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:foo|bar),\s*(?:foo|bar)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "No 'foo, bar' keyword find")

    def setup_testing_render_keywords(self, html='<p>global local subject</p>'):
        self.my_doc.setText(html)
        self.sp.additional_keywords = (('global',))
        self.my_doc.manage_addProperty('qSEO_keywords', ('local'), 'lines')
        aq_inner(self.my_doc).aq_explicit.setSubject('subject')
        html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        return html

    def test_render_only_subject(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=1, settings_use_keywords_lg=1)
        self.html = self.setup_testing_render_keywords()
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="subject"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, "No 'subject' keyword find")

    def test_render_subject_and_local_seokeywords(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=1, settings_use_keywords_lg=2)
        self.html = self.setup_testing_render_keywords()
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:subject|local),\s*(?:subject|local)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "No 'subject, local' keywords find")

    def test_render_only_global_seokeywords(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=2, settings_use_keywords_lg=1)
        self.html = self.setup_testing_render_keywords()
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="global"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, "No 'global' keyword find")

    def test_render_global_and_local_seokeywords(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=2, settings_use_keywords_lg=2)
        self.html = self.setup_testing_render_keywords()
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:global|local),\s*(?:global|local)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "No 'global, local' keywords find")

    def test_render_subject_and_global_seokeywords(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=1)
        self.html = self.setup_testing_render_keywords()
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:subject|global),\s*(?:subject|global)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "No 'subject, global' keywords find")

    def test_render_subject_and_global_and_local_seokeywords(self):
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        self.html = self.setup_testing_render_keywords()
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:subject|global|local),\s*(?:subject|global|local),\s*(?:subject|global|local)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "No 'subject, global, local' keywords find")

    def test_default_values_filter_keywords_by_content_in_configlet(self):
        self.assertEqual(self.sp.getProperty('filter_keywords_by_content', None), True)

    def test_changes_filter_keywords_by_content_in_configlet(self):
        path = self.portal.id+'/@@seo-controlpanel?filter_keywords_by_content=False&form.submitted=1'
        self.publish(path, self.basic_auth)
        self.assertEqual(self.sp.getProperty('filter_keywords_by_content', 0), None)

    def test_filter_keywords_by_content_true(self):
        # Usage metatags keywords, which present in content (filter_keywords_by_content=True)
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2, filter_keywords_by_content=True)
        self.html = self.setup_testing_render_keywords(html='<p>subject global</p>')
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:subject|global),\s*(?:subject|global)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "In metatags keywords not finded \"subject, global\". Checkbox filter_keywords_by_content in configlet is not working.")

    def test_filter_keywords_by_content_false(self):
        # Usage metatags keywords, without taking into consideration presece in content (filter_keywords_by_content=False)
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2, filter_keywords_by_content=False)
        self.html = self.setup_testing_render_keywords(html='<p>subject global</p>')
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="(?:subject|global|local),\s*(?:subject|global|local),\s*(?:subject|global|local)"\s*)){2}/>)',
                    self.html, re.S|re.M)
        self.assert_(m, "In metatags keywords not finded \"subject, global, locals\". Checkbox filter_keywords_by_content in configlet is not working.")

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestUsageKeywords))
    return suite



import re
from base import getToolByName, FunctionalTestCase, newSecurityManager
from config import *

class TestMetaTagsDuplication(FunctionalTestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.basic_auth = 'portal_manager:secret'
        uf = self.app.acl_users
        uf.userFolderAddUser('portal_manager', 'secret', ['Manager'], [])
        user = uf.getUserById('portal_manager')
        if not hasattr(user, 'aq_base'):
            user = user.__of__(uf)
        newSecurityManager(None, user)

        '''Preparation for functional testing'''
        self.my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']
        self.my_doc.update(description="Document description")

    def test_GeneratorMeta(self):
        # Get document without customized canonical url
        abs_path = "/%s" % self.my_doc.absolute_url(1)

        # Before product installation
        self.html = self.publish(abs_path, self.basic_auth).getBody()
        f = re.findall('.*(<meta\s+(?:(?:name="generator"\s*)|(?:content=".*?"\s*)){2}/>)', self.html, re.S|re.M)
        lengen = len(f)
        self.assert_(lengen==1, "There is %d generator meta tag(s) " \
           "before seoptimizer installation" % lengen)

#         # After PROJECT_NAME installation
#         self.qi.installProduct(PROJECT_NAME)
#         html = self.publish(abs_path, self.basic_auth).getBody()
#         lengen = len(regen.findall(html))
#         self.assert_(lengen==1, "There is %d generator meta tag(s) " \
#            "after seoptimizer installation" % lengen)

    def test_DescriptionMeta(self):
        # Get document without customized canonical url
        abs_path = "/%s" % self.my_doc.absolute_url(1)

        # Before product installation
        self.html = self.publish(abs_path, self.basic_auth).getBody()
        f = re.findall('.*(<meta\s+(?:(?:name="description"\s*)|(?:content=".*?"\s*)){2}/>)', self.html, re.S|re.M)
        lendesc = len(f)
        self.assert_(lendesc==1, "There is %d DESCRIPTION meta tag(s) " \
           "before seoptimizer installation" % lendesc)

#         # After PROJECT_NAME installation
#         self.qi.installProduct(PROJECT_NAME)
#         html = self.publish(abs_path, self.basic_auth).getBody()
#         lendesc = len(regen.findall(html))
#         self.assert_(lendesc==1, "There is %d DESCRIPTION meta tag(s) " \
#            "after seoptimizer installation" % lendesc)
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMetaTagsDuplication))
    return suite



from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

def createMultiColumnList(self,slist, numCols, sort_on='title_or_id'):
    try:
        mcl = self.createMultiColumnList(slist, numCols, sort_on=sort_on)
        return mcl
    except AttributeError:
        return [slist]

class SortedDict(dict):
    """ A sorted dictionary.
    """
    security = ClassSecurityInfo()

    security.declarePublic('items')
    def items(self):
        primary_metatags = self.pmt
        lst = [(name,self[name]) for name in primary_metatags                    \
                                                 if name in self.keys()] +       \
              [(name, self[name]) for name in self.keys()                        \
                                                 if name not in primary_metatags]
        return lst


    security.declarePublic('__init__')
    def __init__(self, *args, **kwargs):
        super(SortedDict,self).__init__(*args, **kwargs)
        self.pmt = []


    security.declarePublic('__setitem__')
    def __setitem__(self, i, y):
        super(SortedDict,self).__setitem__(i, y)
        if i not in self.pmt:
            self.pmt.append(i)

    security.declarePublic('pop')
    def pop(self, k, *args, **kwargs):
        super(SortedDict,self).pop(k, *args, **kwargs)
        if k in self.pmt:
            self.pmt.remove(k)

try:
    InitializeClass(SortedDict)
except:
    pass



from zope.interface import Interface

class IMetaKeywords(Interface):
    """Handle the available keywords.
    """
    def getMetaKeywords():
        """Returns all the existing keywords for the current content type.
        """

class IMappingMetaTags(Interface):
    """
    """
    def getMappingMetaTags():
        """Returns mapping {meta_name:accssesor} all the meta tags.
        """



import re
from base import getToolByName, FunctionalTestCase, newSecurityManager
from config import *

class TestExposeDCMetaTags(FunctionalTestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.sp = self.portal.portal_properties.site_properties
        self.qi.installProduct(PROJECT_NAME)
        self.basic_auth = 'portal_manager:secret'
        uf = self.app.acl_users
        uf.userFolderAddUser('portal_manager', 'secret', ['Manager'], [])
        user = uf.getUserById('portal_manager')
        if not hasattr(user, 'aq_base'):
            user = user.__of__(uf)
        newSecurityManager(None, user)

        '''Preparation for functional testing'''
        self.my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']

    def test_exposeDCMetaTags_in_configletOn(self):
        path = self.portal.id+'/@@seo-controlpanel?exposeDCMetaTags=True&form.submitted=1'
        self.publish(path, self.basic_auth)
        self.assert_(self.sp.exposeDCMetaTags)

    def test_exposeDCMetaTags_in_configletOff(self):
        self.publish(self.portal.id+'/@@seo-controlpanel?form.submitted=1', self.basic_auth)
        self.assert_(not self.sp.exposeDCMetaTags)

    def test_exposeDCMetaTagsPropertyOff(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = False)
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m1 = re.match('.*(<meta\s+(?:(?:name="DC.format"\s*)|(?:content=".*?"\s*)){2}/>)', self.html, re.S|re.M)
        m2 = re.match('.*(<meta\s+(?:(?:name="DC.distribution"\s*)|(?:content=".*?"\s*)){2}/>)', self.html, re.S|re.M)
        m = m1 or m2
        self.assert_(not m, 'DC meta tags avaliable when exposeDCMetaTags=False')

    def test_exposeDCMetaTagsPropertyOn(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = True)
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m1 = re.match('.*(<meta\s+(?:(?:name="DC.format"\s*)|(?:content=".*?"\s*)){2}/>)', self.html, re.S|re.M)
        m2 = re.match('.*(<meta\s+(?:(?:name="DC.type"\s*)|(?:content=".*?"\s*)){2}/>)', self.html, re.S|re.M)
        m = m1 and m2
        self.assert_(m, 'DC meta tags not avaliable when createManager=True')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestExposeDCMetaTags))
    return suite



import urllib, re
from cStringIO import StringIO
from base import getToolByName, FunctionalTestCase, newSecurityManager
from config import *

class TestResponse(FunctionalTestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct(PROJECT_NAME)
        self.sp = self.portal.portal_properties.seo_properties
        self.pu = self.portal.plone_utils
        self.basic_auth = 'portal_manager:secret'

        uf = self.app.acl_users
        uf.userFolderAddUser('portal_manager', 'secret', ['Manager'], [])
        user = uf.getUserById('portal_manager')
        if not hasattr(user, 'aq_base'):
            user = user.__of__(uf)
        newSecurityManager(None, user)

        '''Preparation for functional testing'''
        my_doc = self.portal.invokeFactory('Document', id='my_doc')
        my_doc = self.portal['my_doc']
        self.canonurl = 'http://nohost/plone/test.html'
        self.sp.manage_changeProperties(**GLOBAL_CUSTOM_METATAGS)
        self.sp.manage_changeProperties(settings_use_keywords_sg=3, settings_use_keywords_lg=2)
        abs_path = "/%s" % my_doc.absolute_url(1)
        self.form_data = {'seo_description': 'it is description, test keyword1', 'seo_keywords_override:int': 1, 'seo_custommetatags_override:int': 1,
                        'seo_robots_override:int': 1, 'seo_robots': 'ALL', 'seo_description_override:int': 1, 'seo_canonical_override:int': 1,
                        'seo_keywords:list': 'keyword1', 'seo_html_comment': 'no comments',
                        'seo_title_override:int': 1, 'seo_title': 'hello world', 'seo_html_comment_override:int': 1,
                        'seo_distribution_override:int': 1, 'seo_distribution': 'Global', 'seo_canonical': self.canonurl, 'form.submitted:int': 1}
        st = ''
        for d in CUSTOM_METATAGS:
            st += '&seo_custommetatags.meta_name:records=%s&seo_custommetatags.meta_content:records=%s' % (d['meta_name'],d['meta_content'])
        self.publish(path=abs_path+'/@@seo-context-properties', basic=self.basic_auth, request_method='POST', stdin=StringIO(urllib.urlencode(self.form_data)+st))
        #self.publish(abs_path+'/@@seo-context-properties?%s' % urllib.urlencode(self.form_data), self.basic_auth)

        wf_tool = self.portal.portal_workflow
        wf_tool.doActionFor(my_doc, 'publish')

        self.abs_path = abs_path
        self.my_doc = my_doc
        self.html = self.publish(abs_path, self.basic_auth).getBody()

        # now setup page with title equal to plone site's title
        my_doc2 = self.portal.invokeFactory('Document', id='my_doc2')
        my_doc2 = self.portal['my_doc2']
        my_doc2.update(title=self.portal.Title())
        wf_tool.doActionFor(my_doc2, 'publish')
        abs_path2 = "/%s" % my_doc2.absolute_url(1)
        self.html2 = self.publish(abs_path2, self.basic_auth).getBody()

    def testTitle(self):
        m = re.match('.*<title>\\s*hello world\\s*</title>', self.html, re.S|re.M)
        self.assert_(m, 'Title not set in')

    def testTitleDuplication(self):
        """If we are not overriding page title and current page title equals title of the plone site
        then there should be no concatenation of both titles. Only one should be displayed.
        """
        m = re.match('.*<title>\\s*%s\\s*</title>' % self.portal.Title(), self.html2, re.S|re.M)
        self.assert_(m, 'Title is not set correctly, perhaps it is duplicated with plone site title')

    def testDescription(self):
        m = re.match('.*(<meta\s+(?:(?:name="description"\s*)|(?:content="it is description, test keyword1"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, 'Description not set in')

    def testKeywords(self):
        m = re.match('.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="keyword1"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, 'Keywords not set in')

    def testRobots(self):
        m = re.match('.*(<meta\s+(?:(?:name="robots"\s*)|(?:content="ALL"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, 'Robots not set in')

    def testDistribution(self):
        m = re.match('.*(<meta\s+(?:(?:name="distribution"\s*)|(?:content="Global"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, 'Distribution not set in')

    def testHTMLComments(self):
        m = re.match('.*<!--\\s*no comments\\s*-->', self.html, re.S|re.M)
        self.assert_(m, 'Comments not set in')

    def testTagsOrder(self):
        mtop = self.sp.getProperty('metatags_order')
        metatags_order = [t.split(' ')[0] for t in mtop if len(t.split(' '))==2 and t.split(' ')[0] in VIEW_METATAGS]
        m = re.search('.*'.join(['<meta.*name="%s".*/>' %t for t in metatags_order]), self.html, re.S|re.M)
        #m = re.match('.*'.join(['.*(<meta\s+(?:(?:name="%s"\s*)|(?:content=".*"\s*)){2}/>)' %t for t in metatags_order]), self.html, re.S|re.M)
        self.assert_(m, "Meta tags order not supported.")

        mtop = list(mtop)
        mtop.reverse()
        metatags_order = [t.split(' ')[0] for t in mtop if len(t.split(' '))==2 and t.split(' ')[0] in VIEW_METATAGS]
        m = re.search('.*'.join(['<meta.*name="%s".*/>' %t for t in metatags_order]), self.html, re.S|re.M)
        self.assertFalse(m, "Meta tags order not supported.")

        self.sp.manage_changeProperties(**{'metatags_order':tuple(mtop)})
        html = self.publish(self.abs_path, self.basic_auth).getBody()
        m = re.search('.*'.join(['<meta.*name="%s".*/>' %t for t in metatags_order]), self.html, re.S|re.M)
        self.assertFalse(m, "Meta tags order not supported.")

        m = re.search('.*'.join(['<meta.*name="%s".*/>' %t for t in metatags_order]), html, re.S|re.M)
        self.assert_(m, "Meta tags order not supported.")


    def testCustomMetaTags(self):
        for tag in CUSTOM_METATAGS:
            m = re.match('.*(<meta\s+(?:(?:name="%(meta_name)s"\s*)|(?:content="%(meta_content)s"\s*)){2}/>)' % tag, self.html, re.S|re.M)
            if tag['meta_content']:
                self.assert_(m, "Custom meta tag %s not applied." % tag['meta_name'])
            else:
                self.assert_(not m, "Meta tag %s has no content, but is present in the page." % tag['meta_name'])
        m = re.match('.*(<meta\s+(?:(?:name="metatag4"\s*)|(?:content="global_metatag4value"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, "Global custom meta tag %s not applied." % 'metatag4')

    def testDeleteCustomMetaTags(self):
        self.sp.manage_changeProperties(**{'default_custom_metatags':'metatag1|global_metatag1value'})
        my_doc = self.my_doc
        self.form_data = {'seo_custommetatags': CUSTOM_METATAGS,  'seo_custommetatags_override:int': 0, 'form.submitted:int': 1}
        self.publish(path=self.abs_path+'/@@seo-context-properties', basic=self.basic_auth, request_method='POST', stdin=StringIO(urllib.urlencode(self.form_data)))
        self.html = self.publish(self.abs_path, self.basic_auth).getBody()
        m = re.match('.*(<meta\s+(?:(?:name="metatag4"\s*)|(?:content="global_metatag4value"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(not m, "Global custom meta tag %s is prosent in the page." % 'metatag4')
        m = re.match('.*(<meta\s+(?:(?:name="metatag1"\s*)|(?:content="global_metatag1value"\s*)){2}/>)', self.html, re.S|re.M)
        self.assert_(m, "Global custom meta tag %s not applied." % 'metatag1')

    def testCanonical(self):
        m = re.match('.*<link rel="canonical" href="%s" />' % self.canonurl, self.html, re.S|re.M)
        self.assert_(m, self.canonurl)

    def testDefaultCanonical(self):
        """Default canonical url mast add document absolute_url
        """
        # Delete custom canonical url
        my_doc = self.portal['my_doc']
        my_doc._delProperty(id='qSEO_canonical')
        # Get document without customized canonical url
        abs_path = "/%s" % my_doc.absolute_url(1)
        self.html = self.publish(abs_path, self.basic_auth).getBody()

        my_url = my_doc.absolute_url()
        m = re.match('.*<link rel="canonical" href="%s" />' % my_url, self.html, re.S|re.M)
        self.assert_(m, my_url)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestResponse))
    return suite



from Products.CMFCore.permissions import ManagePortal

from quintagroup.seoptimizer.config import *

PROJECT_NAME = 'quintagroup.seoptimizer'

PROPS = {'stop_words':STOP_WORDS, 'fields':FIELDS, 'additional_keywords': []}

CUSTOM_METATAGS = [{'meta_name'    : 'metatag1',
                    'meta_content' : 'metatag1value'},
                   {'meta_name'    : 'metatag2',
                    'meta_content' : 'metatag2value'},
                   {'meta_name'    : 'metatag3',
                    'meta_content' : ''}
                  ]
VIEW_METATAGS = ['DC.creator', 'DC.format', 'DC.date.modified', 'DC.date.created', 'DC.type',
                   'DC.distribution', 'description', 'keywords', 'robots', 'distribution']

GLOBAL_CUSTOM_METATAGS = {'default_custom_metatags':'metatag1|global_metatag1value\nmetatag4|global_metatag4value'}

CONFIGLETS = ({'id':'qSEOptimizer',
    'name':'Search Engine Optimizer',
    'action':'string:${portal_url}/seo-controlpanel',
    'condition':'',
    'category':'Products',
    'visible':1,
    'appId':'qSEOptimizer',
    'permission':ManagePortal},)

qSEO_CONTENT = ['File','Document','News Item']
qSEO_FOLDER  = []
qSEO_TYPES   = qSEO_CONTENT + qSEO_FOLDER



from sets import Set
from DateTime import DateTime 
from Acquisition import aq_inner
from zope.component import queryAdapter
from plone.app.controlpanel.form import ControlPanelView

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone import PloneMessageFactory as pmf

from quintagroup.seoptimizer import SeoptimizerMessageFactory as _

SEPERATOR = '|'
HAS_CANONICAL_PATH = True
SEO_PREFIX = 'seo_'
PROP_PREFIX = 'qSEO_'
SUFFIX = '_override'
PROP_CUSTOM_PREFIX = 'qSEO_custom_'

try:
    from quintagroup.canonicalpath.interfaces import ICanonicalPath
except ImportError:
    HAS_CANONICAL_PATH = False

class SEOContext( BrowserView ):
    """ This class contains methods that allows to edit html header meta tags.
    """
    def getSEOProperty( self, property_name, accessor='' ):
        """ Get value from seo property by property name.
        """
        context = aq_inner(self.context)

        if context.hasProperty(property_name):
            return context.getProperty(property_name)

        if accessor:
            method = getattr(context, accessor, None)
            if not callable(method):
                return None

            # Catch AttributeErrors raised by some AT applications
            try:
                value = method()
            except AttributeError:
                value = None

            return value

    def seo_title( self ):
        """ Generate SEO Title from SEO properties.
        """
        return self.getSEOProperty( 'qSEO_title', accessor='Title' )

    def seo_robots( self ):
        """ Generate SEO Robots from SEO properties.
        """
        robots = self.getSEOProperty( 'qSEO_robots' )
        return robots and robots or 'ALL'

    def seo_description( self ):
        """ Generate Description from SEO properties.
        """

        return self.getSEOProperty( 'qSEO_description', accessor = 'Description')

    def seo_distribution( self ):
        """ Generate Distribution from SEO properties.
        """
        dist = self.getSEOProperty( 'qSEO_distribution' )

        return dist and dist or 'Global'

    def seo_customMetaTags( self ):
        """        Returned seo custom metatags from default_custom_metatags property in seo_properties
        (global seo custom metatags) with update from seo custom metatags properties in context (local seo custom metatags).
        """
        tags = self.seo_globalCustomMetaTags()
        loc = self.seo_localCustomMetaTags()
        names = [i['meta_name'] for i in tags]
        add_tags = []
        for i in loc:
            if i['meta_name'] in names:
                for t in tags:
                    if t['meta_name'] == i['meta_name']:
                        t['meta_content'] = i['meta_content']
            else:
                add_tags.append(i)
        tags.extend(add_tags)
        return tags

    def seo_globalWithoutLocalCustomMetaTags( self ):
        """        Returned seo custom metatags from default_custom_metatags property in seo_properties
        (global seo custom metatags) without seo custom metatags from properties in context (local seo custom metatags).
        """
        glob = self.seo_globalCustomMetaTags()
        loc = self.seo_localCustomMetaTags()
        names = [i['meta_name'] for i in loc]
        tags = []
        for i in glob:
            if i['meta_name'] not in names:
                tags.append(i)
        return tags

    def seo_localCustomMetaTags( self ):
        """ Returned seo custom metatags from properties in context (local seo custom metatags).
        """
        result = []
        property_prefix = 'qSEO_custom_'
        context = aq_inner(self.context)
        for property, value in context.propertyItems():
            if property.startswith(property_prefix) and property[len(property_prefix):]:
                result.append({'meta_name'    : property[len(property_prefix):],
                               'meta_content' : value})
        return result

    def seo_globalCustomMetaTags( self ):
        """ Returned seo custom metatags from default_custom_metatags property in seo_properties.
        """
        result = []
        context = aq_inner(self.context)
        site_properties = getToolByName(context, 'portal_properties')
        if hasattr(site_properties, 'seo_properties'):
            custom_meta_tags = getattr(site_properties.seo_properties, 'default_custom_metatags', [])
            for tag in custom_meta_tags:
                name_value = tag.split(SEPERATOR)
                if name_value[0]:
                    result.append({'meta_name'    : name_value[0],
                                   'meta_content' : len(name_value) == 2 and name_value[1] or ''})
        return result

    def seo_nonEmptylocalMetaTags( self ):
        """
        """
        return bool(self.seo_localCustomMetaTags())

    def seo_html_comment( self ):
        """ Generate HTML Comments from SEO properties.
        """
        html_comment = self.getSEOProperty( 'qSEO_html_comment' )
        return html_comment and html_comment or '' 

    def meta_keywords( self ):
        """ Generate Meta Keywords from SEO properties (global and local) with Subject,
            depending on the options in configlet.
        """
        prop_name = 'qSEO_keywords'
        accessor = 'Subject'
        context = aq_inner(self.context)
        keywords = Set([])
        pprops = getToolByName(context, 'portal_properties')
        sheet = getattr(pprops, 'seo_properties', None)
        method = getattr(context, accessor, None)
        if not callable(method):
            return None

        # Catch AttributeErrors raised by some AT applications
        try:
            subject = Set(method())
        except AttributeError:
            subject = keywords

        if sheet:
          settings_use_keywords_sg = sheet.getProperty('settings_use_keywords_sg')
          settings_use_keywords_lg = sheet.getProperty('settings_use_keywords_lg')
          global_keywords = Set(sheet.getProperty('additional_keywords', None))
          local_keywords = Set(context.getProperty(prop_name, None))
          # Subject overrides global seo keywords and global overrides local seo keywords
          if [settings_use_keywords_sg, settings_use_keywords_lg] == [1, 1]:
              keywords = subject
          # Subject overrides global seo keywords and merge global and local seo keywords
          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [1, 2]:
              keywords = subject | local_keywords
          # Global seo keywords overrides Subject and global overrides local seo keywords
          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [2, 1]:
              keywords = global_keywords
          # Global seo keywords overrides Subject and merge global and local seo keywords
          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [2, 2]:
              keywords = global_keywords | local_keywords
          # Merge Subject and global seo keywords and global overrides local seo keywords
          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [3, 1]:
              keywords = subject | global_keywords
          # Merge Subject and global seo keywords and merge global and local seo keywords
          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [3, 2]:
              keywords = subject | global_keywords | local_keywords
          else:
              keywords = subject
        else:
            keywords = subject

        return tuple(keywords)

    def seo_keywords( self ):
        """ Generate SEO Keywords from SEO properties (global merde local).
        """
        prop_name = 'qSEO_keywords'
        context = aq_inner(self.context)
        keywords = Set([])
        pprops = getToolByName(context, 'portal_properties')
        sheet = getattr(pprops, 'seo_properties', None)

        if sheet:
            settings_use_keywords_sg = sheet.getProperty('settings_use_keywords_sg')
            settings_use_keywords_lg = sheet.getProperty('settings_use_keywords_lg')
            global_keywords = Set(sheet.getProperty('additional_keywords', None))
            local_keywords = Set(context.getProperty(prop_name, None))
            keywords = global_keywords | local_keywords
        else:
            keywords = ''
        return tuple(keywords)

    def seo_canonical( self ):
        """ Generate canonical URL from SEO properties.
        """
        canonical = self.getSEOProperty( 'qSEO_canonical' )

        if not canonical and HAS_CANONICAL_PATH:
            canpath = queryAdapter(self.context, ICanonicalPath)
            if canpath:
                purl = getToolByName(self.context, 'portal_url')()
                cpath = canpath.canonical_path()
                canonical = purl + cpath

        return canonical and canonical or self.context.absolute_url()


class SEOControlPanel( ControlPanelView ):
    """ The class with methods configuration Search Engine Optimizer in configlet.
    """
    template = ViewPageTemplateFile('templates/seo_controlpanel.pt')

    @property
    def portal_properties( self ):
        """
        """
        context = aq_inner(self.context)
        return getToolByName(context, 'portal_properties')

    @property
    def portal_types( self ):
        """ Returned a list of portal types.
        """
        context = aq_inner(self.context)
        return getToolByName(context, 'portal_types')

    def hasSEOAction( self, type_info ):
        """
        """
        return filter(lambda x:x.id == 'seo_properties', type_info.listActions())

    def test( self, condition, first, second ):
        """
        """
        return condition and first or second 

    def getExposeDCMetaTags( self ):
        """ Get value from exposeDCMetaTags property in seo_properties.
        """
        sp = self.portal_properties.site_properties
        return sp.getProperty('exposeDCMetaTags')

    def getDefaultCustomMetatags( self ):
        """ Get values from default_custom_metatags property in seo_properties.
        """
        seo = self.portal_properties.seo_properties
        return seo.getProperty('default_custom_metatags')

    def getMetaTagsOrder( self ):
        """ Get values from metatags_order property in seo_properties.
        """
        seo = self.portal_properties.seo_properties
        return seo.getProperty('metatags_order')

    def getAdditionalKeywords( self ):
        """ Get values from additional_keywords property in seo_properties.
        """
        seo = self.portal_properties.seo_properties
        return seo.getProperty('additional_keywords')

    def getFilterKeywordsByContent( self ):
        """ Get values from filter_keywords_by_content property in seo_properties.
        """
        seo = self.portal_properties.seo_properties
        return seo.getProperty('filter_keywords_by_content')

    def createMultiColumnList( self ):
        """
        """
        context = aq_inner(self.context)
        allTypes = self.portal_types.listContentTypes()
        try:
            return context.createMultiColumnList(allTypes, sort_on='title_or_id')
        except AttributeError:
            return [slist]

    def __call__( self ):
        """ Perform the update and redirect if necessary, or render the page.
        """
        context = aq_inner(self.context)
        request = self.request

        content_types_seoprops_enabled = request.get( 'contentTypes', [] )
        exposeDCMetaTags = request.get( 'exposeDCMetaTags', None )
        additionalKeywords = request.get('additionalKeywords', [])
        default_custom_metatags = request.get('default_custom_metatags', [])
        metatags_order = request.get('metatags_order', [])
        filterKeywordsByContent = request.get('filterKeywordsByContent', None)
        settingsUseKeywordsSG = int(request.get('settingsUseKeywordsSG', 1))
        settingsUseKeywordsLG = int(request.get('settingsUseKeywordsLG', 1))

        site_props = getToolByName(self.portal_properties, 'site_properties')
        seo_props = getToolByName(self.portal_properties, 'seo_properties')

        form = self.request.form
        submitted = form.get('form.submitted', False)

        if submitted:
            site_props.manage_changeProperties(exposeDCMetaTags=exposeDCMetaTags)
            seo_props.manage_changeProperties(additional_keywords=additionalKeywords)
            seo_props.manage_changeProperties(default_custom_metatags=default_custom_metatags)
            seo_props.manage_changeProperties(metatags_order=metatags_order)
            seo_props.manage_changeProperties(content_types_seoprops_enabled=content_types_seoprops_enabled)
            seo_props.manage_changeProperties(filter_keywords_by_content=filterKeywordsByContent)
            seo_props.manage_changeProperties(settings_use_keywords_sg=settingsUseKeywordsSG)
            seo_props.manage_changeProperties(settings_use_keywords_lg=settingsUseKeywordsLG)

            for ptype in self.portal_types.objectValues():
                acts = filter(lambda x: x.id == 'seo_properties', ptype.listActions())
                action = acts and acts[0] or None
                if ptype.getId() in content_types_seoprops_enabled:
                    if action is None:
                        ptype.addAction('seo_properties',
                                        'SEO Properties',
                                        'string:${object_url}/@@seo-context-properties',
                                        "python:exists('portal/@@seo-context-properties')",
                                        'Modify portal content',
                                        'object',
                                        visible=1)
                else:
                    if action !=None:
                        actions = list(ptype.listActions())
                        ptype.deleteActions([actions.index(a) for a in actions if a.getId()=='seo_properties'])
            context.plone_utils.addPortalMessage(pmf(u'Changes saved.'))
            return request.response.redirect('%s/%s'%(self.context.absolute_url(), 'plone_control_panel'))
        else:
            return self.template(contentTypes=content_types_seoprops_enabled, exposeDCMetaTags=exposeDCMetaTags)

    def typeInfo( self, type_name ):
        """ Get info type by type name.
        """
        return self.portal_types.getTypeInfo( type_name )

    def select_settings_use_keywords_sg(self):
        context = aq_inner(self.context)
        site_properties = getToolByName(context, 'portal_properties')
        if hasattr(site_properties, 'seo_properties'):
            settings_use_keywords_sg = getattr(site_properties.seo_properties, 'settings_use_keywords_sg', 0)
        return settings_use_keywords_sg

    def select_settings_use_keywords_lg(self):
        context = aq_inner(self.context)
        site_properties = getToolByName(context, 'portal_properties')
        if hasattr(site_properties, 'seo_properties'):
            settings_use_keywords_lg = getattr(site_properties.seo_properties, 'settings_use_keywords_lg', 0)
        return settings_use_keywords_lg


class SEOContextPropertiesView( BrowserView ):
    """ This class contains methods that allows to manage seo properties.
    """
    template = ViewPageTemplateFile('templates/seo_context_properties.pt')

    def test( self, condition, first, second ):
        """
        """
        return condition and first or second 

    def getMainDomain(self, url):
        """ Get a main domain.
        """
        url = url.split('//')[-1]
        dompath = url.split(':')[0]
        dom = dompath.split('/')[0]
        return '.'.join(dom.split('.')[-2:])

    def validateSEOProperty(self, property, value):
        """ Validate a seo property.
        """
        purl = getToolByName(self.context, 'portal_url')()
        state = ''
        if property == PROP_PREFIX+'canonical':
            # validate seo canonical url property
            pdomain = self.getMainDomain(purl)
            if not pdomain == self.getMainDomain(value):
                state = _('canonical_msg', default=u'Canonical URL mast be in ${pdomain} domain.', mapping={'pdomain': pdomain})
        return state

    def setProperty(self, property, value, type='string'):
        """ Add a new property.

        Sets a new property with the given id, value and type or changes it.
        """
        context = aq_inner(self.context)
        state = self.validateSEOProperty(property, value)
        if not state:
            if context.hasProperty(property):
                context.manage_changeProperties({property: value})
            else:
                context.manage_addProperty(property, value, type)
        return state

    def manageSEOProps(self, **kw):
        """ Manage seo properties.
        """
        context = aq_inner(self.context)
        state = ''
        delete_list, seo_overrides_keys, seo_keys = [], [], []
        seo_items = dict([(k[len(SEO_PREFIX):],v) for k,v in kw.items() if k.startswith(SEO_PREFIX)])
        for key in seo_items.keys():
            if key.endswith(SUFFIX):
                seo_overrides_keys.append(key[:-len(SUFFIX)])
            else:
                seo_keys.append(key)
        for seo_key in seo_keys:
            if seo_key == 'custommetatags':
                self.manageSEOCustomMetaTagsProperties(**kw)
            else:
                if seo_key in seo_overrides_keys and seo_items.get(seo_key+SUFFIX):
                    seo_value = seo_items[seo_key]
                    t_value = 'string'
                    if type(seo_value)==type([]) or type(seo_value)==type(()): t_value = 'lines'
                    state = self.setProperty(PROP_PREFIX+seo_key, seo_value, type=t_value)
                    if state:
                        return state
                elif context.hasProperty(PROP_PREFIX+seo_key):
                    delete_list.append(PROP_PREFIX+seo_key)
        if delete_list:
            context.manage_delProperties(delete_list)
        return state

    def setSEOCustomMetaTags(self, custommetatags):
        """ Set seo custom metatags properties.
        """
        context = aq_inner(self.context)
        for tag in custommetatags:
            self.setProperty('%s%s' % (PROP_CUSTOM_PREFIX, tag['meta_name']), tag['meta_content'])

    def delAllSEOCustomMetaTagsProperties(self):
        """ Delete all seo custom metatags properties.
        """
        context = aq_inner(self.context)
        delete_list = []
        for property, value in context.propertyItems():
            if property.startswith(PROP_CUSTOM_PREFIX)  and not property == PROP_CUSTOM_PREFIX:
                delete_list.append(property)
        if delete_list:
            context.manage_delProperties(delete_list)

    def updateSEOCustomMetaTagsProperties(self, custommetatags):
        """ Update seo custom metatags properties.
        """
        context = aq_inner(self.context)
        site_properties = getToolByName(context, 'portal_properties')
        globalCustomMetaTags = []
        if hasattr(site_properties, 'seo_properties'):
            custom_meta_tags = getattr(site_properties.seo_properties, 'default_custom_metatags', [])
            for tag in custom_meta_tags:
                name_value = tag.split(SEPERATOR)
                if name_value[0]:
                    globalCustomMetaTags.append({'meta_name'    : name_value[0],
                                                 'meta_content' : len(name_value) == 1 and '' or name_value[1]})
        for tag in custommetatags:
            meta_name, meta_content = tag['meta_name'], tag['meta_content']
            if meta_name:
                if not [gmt for gmt in globalCustomMetaTags if (gmt['meta_name']==meta_name and gmt['meta_content']==meta_content)]:
                    self.setProperty('%s%s' % (PROP_CUSTOM_PREFIX, meta_name), meta_content)

    def manageSEOCustomMetaTagsProperties(self, **kw):
        """ Update seo custom metatags properties, if enabled checkbox override or delete properties.

        Change object properties by passing either a mapping object
        of name:value pairs {'foo':6} or passing name=value parameters.
        """
        context = aq_inner(self.context)
        self.delAllSEOCustomMetaTagsProperties()
        if kw.get('seo_custommetatags_override'):
            custommetatags = kw.get('seo_custommetatags', {})
            self.updateSEOCustomMetaTagsProperties(custommetatags)

    def __call__( self ):
        """ Perform the update seo properties and redirect if necessary, or render the page Call method.
        """
        context = aq_inner(self.context)
        request = self.request
        form = self.request.form
        submitted = form.get('form.submitted', False)
        if submitted:
            state = self.manageSEOProps(**form)
            if not state:
                state = _('seoproperties_saved', default=u'Content SEO properties have been saved.')
                context.plone_utils.addPortalMessage(state)
                kwargs = {'modification_date' : DateTime()} 
                context.plone_utils.contentEdit(context, **kwargs) 
                return request.response.redirect(self.context.absolute_url())
            context.plone_utils.addPortalMessage(state, 'error')
        return self.template()



# -*- coding: utf-8 -*-
"""
This module contains the tool of quintagroup.seoptimizer
"""
import os
from setuptools import setup, find_packages


version = '2.3.3'

setup(name='quintagroup.seoptimizer',
      version=version,
      description="Quintagroup Search Engine Optimization Tool",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "INSTALL.txt")).read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),

      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Myroslav Opyr, Volodymyr Romaniuk, Mykola Kharechko, Vitaliy Podoba, Volodymyr Cherepanyak, Taras Melnychuk, Vitaliy Stepanov',
      author_email='support@quintagroup.com',
      url='http://quintagroup.com/services/plone-development/products/qSEOptimizer/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['quintagroup'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from cgi import escape
from zope.component import getMultiAdapter
from Products.CMFPlone.utils import safe_unicode
from plone.app.layout.viewlets.common import ViewletBase
from Products.CMFCore.utils import getToolByName


class TitleCommentViewlet(ViewletBase):
    """ Simple viewlet for custom title rendering.
    """

    def update(self):
        self.portal_state = getMultiAdapter((self.context, self.request),
                                            name=u'plone_portal_state')
        self.context_state = getMultiAdapter((self.context, self.request),
                                             name=u'plone_context_state')
        self.page_title = self.context_state.object_title
        self.portal_title = self.portal_state.portal_title

        self.override_title = self.context.hasProperty('qSEO_title')
        self.override_comments = self.context.hasProperty('qSEO_html_comment')

    def std_title(self):
        portal_title = safe_unicode(self.portal_title())
        page_title = safe_unicode(self.page_title())
        if page_title == portal_title:
            return u"<title>%s</title>" % (escape(portal_title))
        else:
            return u"<title>%s &mdash; %s</title>" % (
                escape(safe_unicode(page_title)),
                escape(safe_unicode(portal_title)))

    def render(self):
        std_title = self.std_title()
        seo_context = getMultiAdapter((self.context, self.request), name='seo_context')
        if not self.override_title:
            if not self.override_comments:
                return std_title
            else:
                qseo_comments = u"<!--%s-->"%safe_unicode(seo_context.seo_html_comment())
                return u"%s\n%s"%(std_title, qseo_comments)
        else:
            qseo_title = u"<title>%s</title>" % safe_unicode(seo_context.seo_title())
            if not self.override_comments:
                return qseo_title
            else:
                qseo_comments = u"<!--%s-->"%safe_unicode(seo_context.seo_html_comment())
                return u"%s\n%s"%(qseo_title, qseo_comments)


class CustomScriptViewlet( ViewletBase ):
    """ Simple viewlet for custom script rendering.
    """
    def getCustomScript( self ):
        context = self.context.aq_inner
        portal_props = getToolByName(context, 'portal_properties')
        seo_props = getToolByName(portal_props, 'seo_properties', None)
        if seo_props is None:
            return '' 
        return seo_props.getProperty('custom_script', '')

    def render( self ):
        return safe_unicode("""%s"""% self.getCustomScript())


class CanonicalUrlViewlet( ViewletBase ):
    """ Simple viewlet for canonical url link rendering.
    """

    def render( self ):
        seo_context = getMultiAdapter((self.context, self.request), name='seo_context')
        return """<link rel="canonical" href="%s" />""" % seo_context.seo_canonical()



from zope.interface import Interface
from plone.theme.interfaces import IDefaultPloneLayer

class IPloneSEOLayer(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 skin layer bound to a Skin
       Selection in portal_skins.
    """

class IValidateSEOKeywordsView(Interface):
    """ View for validating keywords on qSEO_properties_edit_form """

    def validateKeywords(text):
        """ Parse text and validate each keyword (extracted from text) for appearing on the context page """



import urllib, re, os, commands
from xml.dom import minidom, Node

from zope.interface import implements
from Products.Five.browser import BrowserView

from Products.CMFCore.utils import getToolByName

from interfaces import IValidateSEOKeywordsView
from quintagroup.seoptimizer import SeoptimizerMessageFactory as _

class ValidateSEOKeywordsView(BrowserView):

    implements(IValidateSEOKeywordsView)

    def validateKeywords(self, text):
        """ see interface """
        ts = getToolByName(self.context, 'translation_service')
        # extract keywords from text
        if text.lower().strip():
            keywords = map(lambda x: x.strip(), text.lower().strip().split('\n'))
        else:
            return ts.utranslate(None, _(u'Keywords list is empty!'), context=self.context)
        # request html page of context object
        url = '%s?without_metatag_keywords=1' % self.context.absolute_url()

        # extract words from url page using lynx browser (test page by 'url' randered without metatag keywords)
        page_text = commands.getoutput('lynx --dump --nolist %s' % url).lower()
        if page_text and page_text != 'sh: lynx: command not found':
            page_text = page_text.decode('utf8')
        else:
            return _(u'Could not find lynx browser!')

        # check every keyword on appearing in body of html page
        missing = []
        finding = []
        added = {}
        finded = {}
        for keyword in keywords:
            keyword = keyword.decode('utf8')
            if keyword:
                keyword_on_page =  len(re.findall(u'\\b%s\\b' % keyword, page_text, re.I|re.U))
                if keyword not in added.keys() and not keyword_on_page:
                    missing.append(keyword+u' - 0')
                    added[keyword] = 1
                if keyword not in finded.keys() and keyword_on_page:
                    finding.append(keyword+u' - '+repr(keyword_on_page))
                    finded[keyword] = 1
        # return list of missing and fount keywords
        if missing or finding:
            msg = ts.utranslate(None, _('number_keywords', default=u'Number of keywords at page:\n${found}\n${missing}',
                                mapping={'missing':'\n'.join(missing), 'found': '\n'.join(finding)}), context=self.context)
        else:
            msg = ''
        return msg

    def walkTextNodes(self, parent, page_words=[]):
        for node in parent.childNodes:
            if node.nodeType == Node.ELEMENT_NODE:
                self.walkTextNodes(node, page_words)
            elif node.nodeType == Node.TEXT_NODE:
                value = node.nodeValue
                if value is not None:
                    page_words.extend(map(lambda x: x.lower(), value.split()))

    def strip_tags(self, in_text):
        s_list = list(in_text)
        i,j = 0,0

        while i < len(s_list):
            if s_list[i] == '<':
                while s_list[i] != '>':
                    # pop everything from the the left-angle bracket until the right-angle bracket
                    s_list.pop(i)

                # pops the right-angle bracket, too
                s_list.pop(i)
            else:
                i=i+1

        # convert the list back into text
        join_char=''
        return join_char.join(s_list)



import urllib, re
from cStringIO import StringIO
from base import getToolByName, FunctionalTestCase, newSecurityManager
from config import *

class TestBugs(FunctionalTestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct(PROJECT_NAME)
        #self.portal.changeSkin('Plone Default')

        self.basic_auth = 'portal_manager:secret'
        uf = self.app.acl_users
        uf.userFolderAddUser('portal_manager', 'secret', ['Manager'], [])
        user = uf.getUserById('portal_manager')
        if not hasattr(user, 'aq_base'):
            user = user.__of__(uf)
        newSecurityManager(None, user)

    def test_modification_date(self):
        """ Modification date changing on SEO properties edit """
        my_doc = self.portal.invokeFactory('Document', id='my_doc')
        my_doc = self.portal['my_doc']

        md_before = my_doc.modification_date
        abs_path = "/%s" % my_doc.absolute_url(1)
        form_data = {'seo_title': 'New Title',  'seo_title_override:int': 1, 'form.submitted:int': 1}
        self.publish(path=abs_path+'/@@seo-context-properties', basic=self.basic_auth, request_method='POST', stdin=StringIO(urllib.urlencode(form_data)))
        md_after = my_doc.modification_date
        self.assertNotEqual(md_before, md_after)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBugs))
    return suite
